/*
 * Example for TouchKey 2 click
 *
 * Date         Feb 2017
 * Author       Stefan Popovic
 * Copyright    GNU Public License v2
 *
 * Test configuration AVR :
 *  MCU           :        ATMEGA32
 *  Dev. Board    :        EasyAVR v7
 *  SW            :        mikroC PRO for AVR v6.1.2
 *
 *  NOTE          : - Place TouchKey 2 click board on mikroBUS slot 1
 *                  - Connect LCD 
 *                  - This click board sends 2 bytes via UART, on key pressed it
 *                    sends KEY_Byte and on key release it sends 0x00
 *
/*******************************************************************************
* Includes
*******************************************************************************/
#include "built_in.h"
#include <stdbool.h>
#include <stdint.h>
/*******************************************************************************
* Module Preprocessor Constants
*******************************************************************************/
#define KEY_A                0x01
#define KEY_B                0x02
#define KEY_C                0x04
#define KEY_D                0x08
#define KEY_RELEASE          0x00

/*******************************************************************************
* Module Variable Definitions
*******************************************************************************/
sbit TOUCH_KEY_2_RST             at PORTA6_bit;
sbit TOUCH_KEY_2_RST_DIR         at DDA6_bit;
sbit TOUCH_KEY_2_CS             at PORTA5_bit;
sbit TOUCH_KEY_2_CS_DIR         at DDA5_bit;

// LCD module connections
sbit LCD_RS at PORTA2_bit;
sbit LCD_EN at PORTD6_bit;
sbit LCD_D4 at PORTC4_bit;
sbit LCD_D5 at PORTC5_bit;
sbit LCD_D6 at PORTC6_bit;
sbit LCD_D7 at PORTC7_bit;

sbit LCD_RS_Direction at DDA2_bit;
sbit LCD_EN_Direction at DDD6_bit;
sbit LCD_D4_Direction at DDC4_bit;
sbit LCD_D5_Direction at DDC5_bit;
sbit LCD_D6_Direction at DDC6_bit;
sbit LCD_D7_Direction at DDC7_bit;
// End LCD module connections

bool key_A_pressed  = false;
bool key_B_pressed  = false;
bool key_C_pressed  = false;
bool key_D_pressed  = false;
bool key_released   = false;

char key;
char old_key;

/*******************************************************************************
* Function Declarations
*******************************************************************************/
static void system_init( void );
static void display_init( void );
static void check_key_event( void );
/*******************************************************************************
* Function Definitions
*******************************************************************************/

/*
 * System Initialization
 *
 * Configuring MCU ports, initializing UART, enabling interrupt
 * and performing reset
 */
static void system_init( void )
{
    // Configuring RST and CS pins as output
    TOUCH_KEY_2_RST_DIR = 1;
    TOUCH_KEY_2_CS_DIR = 1;

    // UART Initialization
    UART1_Init( 9600 );
    Delay_ms( 100 );

    // UART Interrupt enabling
    UCSRB |= ( 1 << RXCIE );
    SREG  |= ( 1 << SREG_I );

    // Perform TouchKey 2 board reset
    TOUCH_KEY_2_RST = 1;
    Delay_ms( 100 );
    TOUCH_KEY_2_RST = 0;
    Delay_ms( 100 );
    TOUCH_KEY_2_RST = 1;
}

/*
 * LCD Initialization
 *
 * Preparing LCD for work
 */
static void display_init( void )
{
    Lcd_Init();                        // Initialize LCD
    Lcd_Cmd(_LCD_CLEAR);               // Clear display
    Lcd_Cmd(_LCD_CURSOR_OFF);          // Cursor off
    Lcd_Out( 1, 1, "TouchKey 2   AVR" );
    Lcd_Out( 2, 1, "Press any key..." );
}

/*
 * Checking for key pressed events
 *
 * Looking for Key ID and setting appropriate flags. Called inside interrupt
 */
static void check_key_event ( void )
{
    if ( key == KEY_A )
    {
        key_A_pressed = true;
    }
    else if ( key == KEY_B )
    {
        key_B_pressed = true;
    }
     else if ( key == KEY_C )
     {
        key_C_pressed = true;
     }
     else if ( key == KEY_D )
     {
         key_D_pressed = true;
     }
     else if ( key == KEY_RELEASE )
     {
         key_released = true;
     }
}

/*
 * Main function
 */
void main( void )
{
    system_init();
    display_init();

    while( true )
    {

        // Key A has been pressed
        if( key_A_pressed && old_key != KEY_A )
        {
            Lcd_Out( 2, 1, "                " );
            Lcd_Out( 2, 1, "Key A pressed." );
            key_A_pressed = false;
            old_key = KEY_A;
        }
        // Key B has been pressed
        else if( key_B_pressed && old_key != KEY_B )
        {
            Lcd_Out( 2, 1, "                " );
            Lcd_Out( 2, 1, "Key B pressed." );
            key_B_pressed = false;
            old_key = KEY_B;
        }
        // Key C has been pressed
        else if( key_C_pressed && old_key != KEY_C )
        {
            Lcd_Out( 2, 1, "                " );
            Lcd_Out( 2, 1, "Key C pressed." );
            key_C_pressed = false;
            old_key = KEY_C;
        }
        // Key D has been pressed
        else if( key_D_pressed && old_key != KEY_D )
        {
            Lcd_Out( 2, 1, "                " );
            Lcd_Out( 2, 1, "Key D pressed." );
            key_D_pressed = false;
            old_key = KEY_D;
        }
                // Key released
        else if ( key_released && old_key != KEY_RELEASE )
        {
           key_released = false;
           Lcd_Out( 2, 1, "                " );
                   Lcd_Out( 2, 1, "Key released." );
           old_key = KEY_RELEASE;
        }
    }
}

void uart_rx_ISR() iv IVT_ADDR_USART__RXC ics ICS_AUTO 
{
    char tmp = UART_Read();
    key = tmp;
    check_key_event();
}
/************************************************************ END OF FILE *****/